from typing import Optional, Self
from uuid import uuid4, UUID

from portal import db
from .Retrievable import Retrievable
from .InvalidUuidError import InvalidUuidError
from .Uuid import Uuid


class Base(Retrievable):
    """
    A type with a UUID, stored in the database.

    It also has a method to return a dictionary representation for easy serialization.
    """

    # See https://flask-sqlalchemy.palletsprojects.com/en/3.0.x/customizing/#abstract-models-and-mixins
    __abstract__ = True

    uuid = db.Column(Uuid(), primary_key=True, default=uuid4)

    @classmethod
    def get_only(cls, uuid: str | UUID) -> Optional[Self]:
        if isinstance(uuid, str):
            try:
                # Manually convert the uuid to a UUID object (even though sqlalchemy-uuid would do that
                # automatically for us), so we can catch errors.
                uuid_obj = UUID(uuid)
            except ValueError:
                raise InvalidUuidError()
        else:
            uuid_obj = uuid

        return super().get_only(uuid_obj)

    @property
    def primary_key(self) -> UUID:
        return self.uuid
